////////////////////////////////////////////////////////////
//                                //
//                                         //
//                        v 1.4                           //
//                                                        //
//              Copyright (C) 2007 Sagrer                 //
//             LGPL 2.1         //
//                  .  lgpl.txt                     //
//                                                        //
//                  sagrer@yandex.ru                      //
////////////////////////////////////////////////////////////

//      , ....  :
// 1) Sagrer (sagrer@yandex.ru)

////////////////////////////////////////////////////////////////////////

unit KOLForPaths;

interface

uses
  Windows, messages, KOL;

type
  RListOfDirElem = record  // - 1   .
    ElemName : string;     // 
    ElemType : byte;       //  (\)
  end;

  AListOfDirElems = array of RListOfDirElem;    //    RListOfDirElem.

  RListOfDirResult = record       // -  .
    ListOfDirElems : AListOfDirElems;   // .
    Elems : integer;
  end;

const
  ElemType_Dir = 1;
  ElemType_File = 2;
  ElemType_Unknown = 3;

Function GetNearestPath(const Path : string) : string;
Function AddLastSlash(const Path : string) : string;
Function CreateDirFileList(const Path : string; AddFileSize : boolean) : string;
Function PathMinusDir(const Path : string) : string;
Function DelLastSlash(const Path : string) : string;
Function GetDriveTypeName(const DriveName : string): string;
Function GetDriveName(const Path : string) : string;
Function GetVolumeName(RootPath : string) : string;
Function FSObjectExists(const FSObjectName : string) : boolean;
Function CreateListOfDir(const Path : string) : RListOfDirResult;

implementation

Function GetNearestPath(const Path : string) : string;
//      ...

  Function PathMinusOdin(const Path : string) : string;
  // ,   1 . ( 1   ).
  var
    Str1 : string;
    I, J : integer;
    Finded : boolean;

  begin
    Str1 := '';
    J := 0;
    Finded := false;

    //   ""
    For I := Length(Path) downto 1 do begin
      If (Path[I] = '\') and (I <> Length(Path)) then begin
        //   ...
        If Finded = false then begin
          Finded := true;
        end;
      end
      else begin
        If Finded = false then J := J+1;
      end;
    end;

    //   ""

    Str1 := Path;
    DeleteTail(Str1,J);

    Result := Str1;
  end;

  Function FindNearestPath(const Path : string) : string;
  //    ...
  var
    Str1 : string;
  begin
    Str1 := PathMinusOdin(Path);
    If DirectoryExists(Str1) = true then begin
      // ,  ...
      Result := AddLastSlash(Str1);
    end
    else begin
      //  ...
      //     
      Result := FindNearestPath(AddLastSlash(Str1));
    end;
  end;

begin
  //  -    .
  If DirectoryExists(Path[1]+Path[2]+Path[3]) = true then begin
    // .   ,  .
    Result := FindNearestPath(Path);
  end
  else begin
    //   ...
    Result := 'nil';
  end;
end;

Function AddLastSlash(const Path : string) : string;
//  ,   .
begin
  If Path[Length(Path)] <> '\' then begin
    Result := Path+'\';
  end
  else begin
    Result := Path;
  end;
end;

Function CreateDirFileList(const Path : string; AddFileSize : boolean) : string;
var
  FindHandle : THandle;
  FindData : TWin32FindData;
  DirList, FileList, CopList, Fnam, ResCopList : string;
  Bool1 : Boolean;

begin
  //  - ,   
  //   (path) .
  //    ...  
  //   :).

  //  -  .
  DirList := '';
  FileList := '';
  CopList := '';
  FNam := '';
  ResCopList := '';
  // -  .   -     .
  FindData.dwFileAttributes := FILE_ATTRIBUTE_NORMAL;
  FindHandle := FindFirstFile(PChar(Path+'*.*'),FindData);
  if FindHandle <> INVALID_HANDLE_VALUE then begin
    repeat
      FNam := FindData.cFileName;
      //MsgOK(FindData.cFileName+'     '+Int2Str(FindData.dwFileAttributes));
      if (FNam <> '..') and (FNam <> '.') then begin
        //If DirectoryExists(Path+FNam) = true then begin
        If FindData.dwFileAttributes and FILE_ATTRIBUTE_DIRECTORY <> 0 then begin
          //    -  .
          DirList := DirList+'DIR|';
          DirList := DirList+FNam+'|';
        end
        else begin
          FileList := FileList+'FILE|';
          FileList := FileList+FNam+'|';
          //   .
          If AddFileSize = true then begin
            FileList := FileList+Int2Str(FileSize(Path+FNam))+'|';
          end;  
        end;
      end;
      Bool1 := FindNextFile(FindHandle,FindData);
    until Bool1 = false;
    FindClose(FindHandle);
  end;
  //  .
  // -    .
  If DirList <> '' then begin
    repeat
      FNam := Parse(DirList,'|');
      CopList := CopList+FNam+'|';
      FNam := Parse(DirList,'|');
      CopList := CopList+FNam+'|';
      ResCopList := CreateDirFileList(Path+FNam+'\',AddFileSize);
      If ResCopList <> '' then begin
        //     -.
        CopList := CopList+'INDIR|'+ResCopList+'ENDDIR|';
      end;
    until DirList = '';
  end;
  //  .
  If FileList <> '' then begin
    repeat
      FNam := Parse(FileList,'|');
      CopList := CopList+FNam+'|';
      FNam := Parse(FileList,'|');
      CopList := CopList+FNam+'|';
      If AddFileSize = true then begin
        FNam := Parse(FileList,'|');
        CopList := CopList+FNam+'|';
      end;  
    until FileList = '';
  end;
  //  "" -   .
  Result := CopList;
end;

Function PathMinusDir(const Path : string) : string;
var
  str1, str2, str3 : string;
  I : integer;

begin
  // ,   Path  1 
  //  .

  //  .
  str1 := ExcludeTrailingPathDelimiter(Path);

  //     -   
  //  "".
  I := 0;
  str3 := '';
  repeat
    str2 := '';
    repeat
      I := I+1;
      if (str1[I] <> '\') or (str1[I] <> #0) then str2 := str2 + str1[I];
    until (str1[I] = '\') or (str1[I] = #0);
    If str1[I] <> #0 then str3 := str3 + str2;
  until str1[I] = #0;

  //   - str3

  Result := str3;
end;

Function DelLastSlash(const Path : string) : string;
//    .
var
  I : integer;
begin
  If Length(Path) > 0 then begin
    If Path[Length(Path)] = '\' then begin
      Result := '';
      For I := 1 to Length(Path)-1 do begin
        Result := Result+Path[I];
      end;
    end
    else begin
      Result := Path;
    end;
  end
  else begin
    Result := Path;
  end;
end;

Function GetDriveTypeName(const DriveName : string): string;
var
  DriveType: UInt;
begin
  //   
  DriveType:=GetDriveType(PChar(DriveName));
  case DriveType of
    0:               Result:='cant be determined';
    1:               Result:='does not exists';
    DRIVE_REMOVABLE: Result:='removable';
    DRIVE_FIXED:     Result:='fixed';
    DRIVE_REMOTE:    Result:='remote';
    DRIVE_CDROM:     Result:='cd-rom';
    DRIVE_RAMDISK:   Result:='ramdisk';
    else
      Result:='unknown'
  end;
end;

Function GetDriveName(const Path : string) : string;
begin
  //   3   .
  If Length(Path) >= 3 then begin
    //   3 
    Result := Path[1]+Path[2]+Path[3];
  end
  else begin
    // 3   -   
    Result := '';
  end;
end;

Function GetVolumeName(RootPath : string) : string;
var
  VNBuffer : array[0..11] of char;
  MCLength, FSFlags : DWORD;
begin
  //    .
  Result:='';
  if GetVolumeInformation(PChar(RootPath),VNBuffer,SizeOf(VNBuffer),nil,MCLength,FSFlags,nil,0) then begin
    Result:=string(VNBuffer);
  end;  
end;

Function FSObjectExists(const FSObjectName : string) : boolean;
begin
  //,    ,     
  //    - ,    .

  Result := false;
  If (FileExists(FSObjectName) = true) or (DirectoryExists(FSObjectName) = true) then begin
    Result := true;
  end;
end;

Function CreateListOfDir(const Path : string) : RListOfDirResult;
var
  FindHandle : THandle;
  FindData : TWin32FindData;
  I : integer;
  Inserted : boolean;

  Procedure InsertArrItem(const Index : integer; Text : string; ItemType : integer);
  var
    I : integer;
  begin
    //    ,      1.
    Result.Elems := Result.Elems+1;
    SetLength(Result.ListOfDirElems,Result.Elems);
    if Index > Result.Elems-2 then begin
      //     .
      Result.ListOfDirElems[Result.Elems-1].ElemName := FindData.cFileName;
      Result.ListOfDirElems[Result.Elems-1].ElemType := ItemType;
    end
    else begin
      //    .
      for I := Result.Elems-1 downto Index+1 do begin
        Result.ListOfDirElems[I] := Result.ListOfDirElems[I-1]
      end;

      //   -    :).
      Result.ListOfDirElems[Index].ElemName := FindData.cFileName;
      Result.ListOfDirElems[Index].ElemType := ItemType;
    end;
  end;

begin
  // -       
  //  Path     -    .
  //       ListOfDirElems.

  SetLength(Result.ListOfDirElems,0);   //  0
  Result.Elems := 0;      // 0.

  // -  .   -     .
  //    .
  If DirectoryExists(Path) = true then begin
    FindData.dwFileAttributes := FILE_ATTRIBUTE_NORMAL;
    FindHandle := FindFirstFile(PChar(Path+'*.*'),FindData);
    if FindHandle <> INVALID_HANDLE_VALUE then begin
      repeat
        // .
        //     .
        // -    .
        //  -  .

        If FindData.dwFileAttributes and FILE_ATTRIBUTE_DIRECTORY <> 0 then begin
          //    -  .

          // .
          I := -1;
          Inserted := false;
          if Result.Elems <> 0 then begin
            repeat
              I := I+1;
              if Result.ListOfDirElems[i].ElemType = ElemType_Dir then begin
                //   .
                if AnsiCompareText(FindData.cFileName,Result.ListOfDirElems[i].ElemName) < 0 then begin
                  //,      .
                  InsertArrItem(I,FindData.cFileName,ElemType_Dir);
                  Inserted := true;
                end;
              end;
            until (Inserted = true) or
            (Result.ListOfDirElems[i].ElemType <> ElemType_Dir) or
            (I = Result.Elems-1);
            if Result.ListOfDirElems[i].ElemType = ElemType_File then begin
              //   :).
              i := i-1;
            end;
          end;

          if Inserted = false then begin
            //  .
            //   -    .
            InsertArrItem(I+1,FindData.cFileName,ElemType_Dir);
            Inserted := true;
          end;
        end
        else begin
          //    - .
          // .
          Inserted := false;
          if Result.Elems <> 0 then begin
            //  .
            I := -1;
            repeat
              I := I+1;
            until (Result.ListOfDirElems[i].ElemType = ElemType_File) or
            (I = Result.Elems-1);

            //    - .
            if Result.ListOfDirElems[i].ElemType = ElemType_File then begin
              //  .
              I := I-1;
              repeat
                I := I+1;
                if Result.ListOfDirElems[i].ElemType = ElemType_File then begin
                  //   .
                  if AnsiCompareText(FindData.cFileName,Result.ListOfDirElems[i].ElemName) < 0 then begin
                    //,      .
                    InsertArrItem(I,FindData.cFileName,ElemType_File);
                    Inserted := true;
                  end;
                end;
              until (Inserted = true) or
              (Result.ListOfDirElems[i].ElemType <> ElemType_File) or
              (I = Result.Elems-1);
            end;
          end;

          if Inserted = false then begin
            //  .
            //   -    .
            InsertArrItem(Result.Elems,FindData.cFileName,ElemType_File);
            Inserted := true;
          end;
        end;

        //   - -  ,  ....     .
        if Inserted = false then begin
          InsertArrItem(Result.Elems-1,FindData.cFileName,ElemType_Unknown);
        end;

      until FindNextFile(FindHandle,FindData) = false;
      FindClose(FindHandle);
    end;
  end;  
end;

end.
