///////////////////////////////////////////////////////////
//                        EiEdit                         //
//                   //
//             Copyright (C) 2002 Gipat Group            //
//                             //
//    Gipat Group's opened EI-editor-utility license     //
//                       1.0                       //
//                                                       //
//                  www.gipatgroup.org                   //
///////////////////////////////////////////////////////////

//      , ....  :
// 1) Sagrer (sagrer@yandex.ru)

////////////////////////////////////////////////////////////////////////

unit TextEdForm;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ComCtrls, ImgList, shellapi, KOLForPaths,
  KOLFileInString;

type
  TTextEdF = class(TForm)
    Label1: TLabel;
    EditBox: TRichEdit;
    FiltersCB: TComboBox;
    AddFilterBut: TButton;
    SaveBut: TButton;
    RaspakBut: TButton;
    CloseBut: TButton;
    DelFilterBut: TButton;
    FileListBox: TListView;
    ImageList1: TImageList;
    PathEd: TEdit;
    UsePathBut: TButton;
    FileNameEd: TEdit;
    _1ClickCB: TCheckBox;
    UseCritSizCB: TCheckBox;
    CritSizED: TEdit;
    Label2: TLabel;
    ZapakBut: TButton;
    WordWrapCB: TCheckBox;
    GroupBox1: TGroupBox;
    RtfModeRB: TRadioButton;
    TxtModeRB: TRadioButton;
    procedure CloseButClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FormCreate(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure FiltersCBClick(Sender: TObject);
    procedure FiltersCBKeyPress(Sender: TObject; var Key: Char);
    procedure AddFilterButClick(Sender: TObject);
    procedure DelFilterButClick(Sender: TObject);
    procedure UsePathButClick(Sender: TObject);
    procedure PathEdKeyPress(Sender: TObject; var Key: Char);
    procedure FileListBoxDblClick(Sender: TObject);
    procedure _1ClickCBClick(Sender: TObject);
    procedure UseCritSizCBClick(Sender: TObject);
    procedure CritSizEDExit(Sender: TObject);
    procedure FileListBoxSelectItem(Sender: TObject; Item: TListItem;
      Selected: Boolean);
    procedure SaveButClick(Sender: TObject);
    procedure RaspakButClick(Sender: TObject);
    procedure ZapakButClick(Sender: TObject);
    procedure WordWrapCBClick(Sender: TObject);
    procedure TxtModeRBClick(Sender: TObject);
    procedure RtfModeRBClick(Sender: TObject);
    procedure FileListBoxEdited(Sender: TObject; Item: TListItem;
      var S: String);
    procedure FileListBoxKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure FileListBoxEditing(Sender: TObject; Item: TListItem;
      var AllowEdit: Boolean);
    procedure FileListBoxExit(Sender: TObject);
  private
    { Private declarations }
    FirstShowed : boolean;   //,        ...
    Showing : boolean;
    FLB_Editing : boolean;
    Function GetFilterFull(const Index : integer) : string;
    Function GetFilter(const Index : integer) : string;
    Function GetFilterDesc(const Index : integer) : string;
  public
    { Public declarations }
    FiltersCount : integer;        //    .
    CurrPath : string;             //  .
    FilterNow : string;
    OpenedTxt : string;       //  .
    ListOfDirResult : RListOfDirResult;  //   .
    NumInArr : integer;       // .
    Function CheckFilterExists(const Filter : string) : integer;
    Procedure SetCurrPath(const ThePath : string);
    Procedure ShowFileList;
    Function GetTypeOfFile(const FileName : string) : string;
    Procedure OpenSelectedTxt;
    Procedure OpenSelectedRtf;
  end;


var
  TextEdF: TTextEdF;
  TextEdForms : array of TTextEdF;
  TextEdF_Quantity : integer;
  CurrNumInArr : integer;

implementation
uses initer, KOL, MySys, NewFilterAskForm, PZed, DelF;

{$R *.dfm}

procedure TTextEdF.CloseButClick(Sender: TObject);
begin
  Close;
end;

procedure TTextEdF.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  Action := caFree;    //     .
  MainForm.TextEditMenu.Checked := false;

  //        :)
  TxtEdDlgCritSiz := Str2Int(CritSizED.Text);
  AnIniFil.Load(IniFilPath);  // .
  AnIniFil.WriteInteger('MainSettings','TxtEdDlgCritSiz',TxtEdDlgCritSiz);
  AnIniFil.Save(IniFilPath);  // .    :)

  //  -         :).
  TextEdForms[NumInArr] := nil;
end;

procedure TTextEdF.FormCreate(Sender: TObject);
begin
  FirstShowed := false;
  Showing := false;
  FilterNow := '';
  NumInArr := CurrNumInArr;
end;

procedure TTextEdF.FormShow(Sender: TObject);
var
  I : integer;
  TempFilters, TempStr1 : string;
  
begin
  If FirstShowed = false then begin
    //    ...
    FirstShowed := true;
    Showing := true;

    //.
    Caption := '   '+IntToStr(NumInArr+1);

    //   ...
    FiltersCount := (SimvCountInStr(TxtEdDlgFiltList,'|')+1) div 2;  // .
    If FiltersCount > 0 then begin
      //    ...
      TempFilters := TxtEdDlgFiltList;  //    . .

      //   ...
      For I := 1 to FiltersCount do begin
        //   ...
        TempStr1 := Parse(TempFilters,'|');
        TempStr1 := TempStr1+'    |    '+Parse(TempFilters,'|');

        //    ...
        FiltersCB.Items.Add(TempStr1);
      end;
    end;

    //   ...
    If (TxtEdDlgActFilt-1 <= FiltersCB.Items.Capacity) and (TxtEdDlgActFilt-1 >= 0) then begin
      FiltersCB.ItemIndex := TxtEdDlgActFilt-1;
      FiltersCBClick(Sender);
    end;

    // -   " ".
    SetCurrPath(TextsEdStart);
    ShowFileList;

    //      -.
    _1ClickCB.Checked := TxtEdDlg1Click;
    UseCritSizCB.Checked := TxtEdDlgUseCritSiz;
    CritSizED.Text := Int2Str(TxtEdDlgCritSiz);
    WordWrapCB.Checked := TxtEdDlgWordWrap;
    Showing := false;

    //  WordWrap .
    EditBox.WordWrap := TxtEdDlgWordWrap;

    //        :).
    FLB_Editing := false;
  end;
end;

Function TTextEdF.GetFilterFull(const Index : integer) : string;
var
  TempFilters : string;
  I, J : integer;
begin
  //      .

  Result := '';
  If (Index <= FiltersCount) and (Index > 0) then begin
      //    ...
      If FiltersCount > 0 then begin
      //    ...
      TempFilters := TxtEdDlgFiltList;  //    . .

      //   ...
      For I := 1 to Index do begin
        //   ...
        Result := Parse(TempFilters,'|');
        Result := Result+'|'+Parse(TempFilters,'|');
      end;
    end;
  end;
end;

Function TTextEdF.GetFilter (const Index : integer) : string;
var
  TempStr : string;
begin
  //     -.
  TempStr := GetFilterFull(Index);
  Result := Parse(TempStr,'|');
end;

Function TTextEdF.GetFilterDesc (const Index : integer) : string;
var
  TempStr : string;
begin
  //     -.
  TempStr := GetFilterFull(Index);
  Parse(TempStr,'|');
  Result := TempStr;
end;

procedure TTextEdF.FiltersCBClick(Sender: TObject);
var
  TempStr : string;
begin
  //ShowMessage('Clicked!!!');
  //    -        :).
  TxtEdDlgActFilt := FiltersCB.ItemIndex+1;
  AnIniFil.Load(IniFilPath);  // .
  AnIniFil.WriteInteger('MainSettings','TxtEdDlgActFilt',TxtEdDlgActFilt);
  AnIniFil.Save(IniFilPath);  // .    :)

  // -    .
  TempStr := FiltersCB.Text;
  FilterNow := FilterValue(Parse(TempStr,'|'));

  //  .
  ShowFileList;
end;

procedure TTextEdF.FiltersCBKeyPress(Sender: TObject; var Key: Char);
var
  TempStr : string;
  TempInt : Integer;
begin
  // -    :).
  If Key = #13 then begin
    //Enter
    //   -    :).

    //   - -    .
    TempStr := FiltersCB.Text;
    FilterNow := FilterValue(Parse(TempStr,'|'));

    //  -     .  
    // -    .
    TempInt := CheckFilterExists(FilterNow);
    If TempInt <> 0 then begin
      //  , .  .
      FiltersCB.ItemIndex := TempInt-1;
      
      //   -    .
      TxtEdDlgActFilt := TempInt;
      AnIniFil.Load(IniFilPath);  // .
      AnIniFil.WriteInteger('MainSettings','TxtEdDlgActFilt',TxtEdDlgActFilt);
      AnIniFil.Save(IniFilPath);  // .    :)
    end;

  end;
end;

Function TTextEdF.CheckFilterExists(const Filter : string) : integer;
var
  I : integer;
begin
  // ,     .
  //  -  ,   -   .

  Result := 0;
  For I := 1 to FiltersCount do begin
    If AnsiUpperCase(GetFilter(I)) = AnsiUpperCase(Filter) then Result := I;
  end;
end;

procedure TTextEdF.AddFilterButClick(Sender: TObject);
var
  TheFilter : string;
begin
  //  .   ,    
  //  ...      ...

  TheFilter := FilterValue(FiltersCB.Text);
  //  .   .
  // - ,     
  //   -    .
  //  -      .

  If CheckFilterExists(TheFilter) <> 0 then begin
    //     
    NewFilterAskF := TNewFilterAskF.Create(Application);
    NewFilterAskF.FilterEd.Text := '';
    NewFilterAskF.FilterDescEd.Text := '';
  end
  else begin
    //      ...
    NewFilterAskF := TNewFilterAskF.Create(Application);
    NewFilterAskF.FilterEd.Text := TheFilter;
    NewFilterAskF.FilterDescEd.Text := '';
  end;

  // ...
  NewFilterAskF.ShowModal;

  //    -     :).
  If NewFilterAskF.ModalResult = mrOk then begin
    //    ...
    If TxtEdDlgFiltList <> '' then begin
      TxtEdDlgFiltList := TxtEdDlgFiltList+'|'+NewFilterAskF.FilterEd.Text+'|'+NewFilterAskF.FilterDescEd.Text;
    end
    else begin
      //        :).
      TxtEdDlgFiltList := NewFilterAskF.FilterEd.Text+'|'+NewFilterAskF.FilterDescEd.Text;
    end;
    //   :).
    AnIniFil.Load(IniFilPath);  // .
    AnIniFil.WriteString('MainSettings','TxtEdDlgFiltList',TxtEdDlgFiltList);
    AnIniFil.Save(IniFilPath);  // .    :)
    //       ,  .
    FiltersCount := FiltersCount+1;
    FiltersCB.Items.Add(GetFilter(FiltersCount)+'    |    '+GetFilterDesc(FiltersCount));
    FiltersCB.ItemIndex := FiltersCount-1;
    FiltersCBClick(Sender);
  end;

  // ...
  NewFilterAskF.Free;
end;

procedure TTextEdF.DelFilterButClick(Sender: TObject);
var
  FiltNum, I : integer;
  TheFilter, ResultFilt : string;
begin
  //     .
  If MessageBox(Application.Handle,'    ?','!',MB_YESNO or MB_ICONWARNING) = ID_YES then begin
    //   -      , 
    //   .
    TheFilter := FilterValue(FiltersCB.Text);
    FiltNum := CheckFilterExists(TheFilter);

    If FiltNum <> 0 then begin
      //      .
      ResultFilt := '';
      For I := 1 to FiltersCount do begin
        //   
        If I <> FiltNum then begin
          //       -      .
          If ResultFilt = '' then begin
            ResultFilt := GetFilterFull(I);
          end
          else begin
            ResultFilt := ResultFilt+'|'+GetFilterFull(I);
          end;  
        end;
      end;

      // -        .
      TxtEdDlgFiltList := ResultFilt;
      AnIniFil.Load(IniFilPath);  // .
      AnIniFil.WriteString('MainSettings','TxtEdDlgFiltList',TxtEdDlgFiltList);
      AnIniFil.Save(IniFilPath);  // .    :)
      //       ,  .
      FiltersCount := FiltersCount-1;
      FiltersCB.Items.Delete(FiltNum-1);
      If FiltersCount <> 0 then begin
        //     -     
        FiltersCB.ItemIndex := 0;
        FiltersCBClick(Sender);
      end
      else begin
        //    -   .
        FiltersCB.Clear;
        FiltersCBClick(Sender);
      end;
    end;

  end;  
end;

Procedure TTextEdF.SetCurrPath(const ThePath : string);
begin
  //     +   .
  If DirectoryExists(ThePath) = true then begin
      CurrPath := ThePath;
  end
  else begin
    CurrPath := GetNearestPath(ThePath);
  end;
  PathEd.Text := CurrPath;
  ListOfDirResult := CreateListOfDir(CurrPath);
end;

Procedure TTextEdF.ShowFileList;
var
  I : integer;
  LI : TListItem;
  TypeOfFile : string;

begin
  //          .

  //  ...
  RaspakBut.Enabled := false;
  // ...
  ZapakBut.Enabled := false;
  
  If ListOfDirResult.Elems <> 0 then begin
    //        
    //    ,       :).
    //  -         .
    FileListBox.Clear;

    // ,  .
    For I := 0 to ListOfDirResult.Elems-1 do begin
      If ((ListOfDirResult.ListOfDirElems[I].ElemName <> '.') and (StrSatisfy(ListOfDirResult.ListOfDirElems[I].ElemName,FilterNow))) or (ListOfDirResult.ListOfDirElems[I].ElemName = '..') or ( (ListOfDirResult.ListOfDirElems[I].ElemType = ElemType_Dir) and (ListOfDirResult.ListOfDirElems[I].ElemName <> '.')) then begin
        LI := FileListBox.Items.Add;
        LI.Caption := ListOfDirResult.ListOfDirElems[I].ElemName;
        //  ...

        If ListOfDirResult.ListOfDirElems[I].ElemType = ElemType_Unknown then begin
          //.
          LI.ImageIndex := ElemType_Unknown;
        end;

        If ListOfDirResult.ListOfDirElems[I].ElemType = ElemType_Dir then begin
          //
          //    .
          If (ListOfDirResult.ListOfDirElems[I].ElemName <> '.') and (ListOfDirResult.ListOfDirElems[I].ElemName <> '..') then begin
            LI.ImageIndex := ElemType_Dir;
          end
          else begin
            LI.ImageIndex := 0;
          end;
        end;

        If ListOfDirResult.ListOfDirElems[I].ElemType = ElemType_File then begin
          //
          TypeOfFile := GetTypeOfFile(ListOfDirResult.ListOfDirElems[I].ElemName);

          If TypeOfFile = 'unknown' then begin
            LI.ImageIndex := ElemType_File;
          end;

          If TypeOfFile = 'resbuild_res' then begin
            LI.ImageIndex := 4;
          end;

          If TypeOfFile = 'rtf' then begin
            LI.ImageIndex := 5;
          end;
        end;

      end;
    end;
  end;
end;

procedure TTextEdF.UsePathButClick(Sender: TObject);
begin
  //   -      .
  PathEd.Text := AddLastSlash(PathEd.Text);
  SetCurrPath(PathEd.Text);
  ShowFileList;
end;

procedure TTextEdF.PathEdKeyPress(Sender: TObject; var Key: Char);
begin
  If Key = #13 then begin
    //Enter.
    UsePathButClick(Sender);
  end;
end;

procedure TTextEdF.FileListBoxDblClick(Sender: TObject);
var
  Done : boolean;  //    .
  RBResult : string;

begin
  //   .
  Done := false;
  If FileListBox.Selected <> nil then begin
    If FileListBox.Selected.ImageIndex = 0 then begin
      //0 -    .    .
      SetCurrPath(PathMinusDir(CurrPath));
      ShowFileList;
      Done := true;
    end;

    If Done = false then begin
      If FileListBox.Selected.ImageIndex = 1 then begin
        //1 -    .    .
        SetCurrPath(CurrPath+ FileListBox.Selected.Caption+'\');
        ShowFileList;
        Done := true;
      end;
    end;

    If Done = false then begin
      If FileListBox.Selected.ImageIndex = 2 then begin
        //2 -     .
        //     :).  -   .
        If TxtEdDlgUseCritSiz = true then begin
          //     -  :).
          If TxtEdDlgCritSiz > FileSize(CurrPath+FileListBox.Selected.Caption) then begin
            //     -   .
            OpenSelectedTxt;
          end;
        end
        else begin
          //  .  .
          OpenSelectedTxt;
        end; 
        Done := true;
      end;
    end;

    If Done = false then begin
      If FileListBox.Selected.ImageIndex = 4 then begin
        //4 -    .
        //  .
        RBResult := UnPackResDialog(CurrPath+FileListBox.Selected.Caption);
        If RBResult <> '' then begin
          //   .
          //  -.
          SetCurrPath(RBResult);
          ShowFileList;
        end;
        Done := true;
      end;
    end;

    If Done = false then begin
      If FileListBox.Selected.ImageIndex = 5 then begin
        //5 -  rtf.
        // -  .
        OpenSelectedRtf;
        Done := true;
      end;
    end;

  end;  
end;

procedure TTextEdF._1ClickCBClick(Sender: TObject);
begin
  TxtEdDlg1Click := _1ClickCB.Checked;
  If Showing = false then begin
    //     
    //"" .
    AnIniFil.Load(IniFilPath);  // .
    AnIniFil.WriteBool('MainSettings','TxtEdDlg1Click',TxtEdDlg1Click);
    AnIniFil.Save(IniFilPath);  // .    :)
  end;
end;

procedure TTextEdF.UseCritSizCBClick(Sender: TObject);
begin
  TxtEdDlgUseCritSiz := UseCritSizCB.Checked;
  If Showing = false then begin
    //     
    //"" .
    AnIniFil.Load(IniFilPath);  // .
    AnIniFil.WriteBool('MainSettings','TxtEdDlgUseCritSiz',TxtEdDlgUseCritSiz);
    AnIniFil.Save(IniFilPath);  // .    :)
  end;
end;

procedure TTextEdF.CritSizEDExit(Sender: TObject);
begin
  TxtEdDlgCritSiz := Str2Int(CritSizED.Text);
  AnIniFil.Load(IniFilPath);  // .
  AnIniFil.WriteInteger('MainSettings','TxtEdDlgCritSiz',TxtEdDlgCritSiz);
  AnIniFil.Save(IniFilPath);  // .    :)
end;

Function TTextEdF.GetTypeOfFile(const FileName : string) : string;
begin
  //      :).
  Result := 'unknown';   //

  If (StrSatisfy(ExtractFileName(FileName),'*.res')) or (StrSatisfy(ExtractFileName(FileName),'*.mq')) then begin
    //
    Result := 'resbuild_res';
  end;

  If StrSatisfy(ExtractFileName(FileName),'*.rtf') then begin
    //.
    Result := 'rtf';
  end;
end;

procedure TTextEdF.FileListBoxSelectItem(Sender: TObject; Item: TListItem;
  Selected: Boolean);
begin
  //  :).

  //    -1  :).
  If FileListBox.ItemIndex <> -1 then begin

    //    .

    // -    .
    If FileListBox.Selected.ImageIndex = 4 then begin
      //4 -    .
      //  .
      RaspakBut.Enabled := true;
    end
    else begin
      If RaspakBut.Enabled = true then RaspakBut.Enabled := false;
    end;

    If (FileListBox.Selected.ImageIndex = 1) or (FileListBox.Selected.ImageIndex = 0) then begin
      //1 -  .
      //0 -      -      :).
      //  .
      ZapakBut.Enabled := true;
    end
    else begin
      If ZapakBut.Enabled = true then ZapakBut.Enabled := false;
    end;

    //    .

    //      -   :).
    If TxtEdDlg1Click = true then begin
      If FileListBox.Selected.ImageIndex = 2 then begin
        //2 -     .
        //     :).  -   .
        FileListBoxDblClick(Sender);  //    :).
      end;
    end;

  end;
    
end;

Procedure TTextEdF.OpenSelectedTxt;
begin
  //   .
  OpenedTxt := CurrPath+FileListBox.Selected.Caption;
  If FSObjectExists(OpenedTxt) = true then begin
    // . .  -     .
    FileNameEd.Text := FileListBox.Selected.Caption;
    EditBox.Clear;
    EditBox.PlainText := true;
    TxtModeRB.Checked := true;
    EditBox.Lines.LoadFromFile(OpenedTxt);
  end;
end;

procedure TTextEdF.SaveButClick(Sender: TObject);
var
  AllOk : boolean;

begin
  //   -  .

  AllOk := false;
  If OpenedTxt = CurrPath+FileNameEd.Text then begin
    //       .   .
    AllOk := true;
  end
  else begin
    // .      -     .
    If FSObjectExists(CurrPath+FileNameEd.Text) = true then begin
      If MessageBox(Application.Handle,PChar(' '+#13+CurrPath+FileNameEd.Text+#13+'?'),'!',MB_YESNO or MB_ICONWARNING) = ID_YES then begin
        AllOk := true;  // .
      end;
    end
    else begin
      AllOk := true;
    end;
  end;

  If AllOk = true then begin
    // .
    OpenedTxt := CurrPath+FileNameEd.Text;
    EditBox.Lines.SaveToFile(OpenedTxt);
    //  -  .
    UsePathButClick(Sender);    //  :).
  end;
end;

procedure TTextEdF.RaspakButClick(Sender: TObject);
begin
  FileListBoxDblClick(Sender);  //    :).
end;

procedure TTextEdF.ZapakButClick(Sender: TObject);
var
  RBResult : string;
  
begin
  //   -    
  //   .
  if FileListBox.Selected.ImageIndex = 1 then begin
    //    .
    RBResult := PackResDialog(CurrPath+FileListBox.Selected.Caption+'\');
  end;
  if FileListBox.Selected.ImageIndex = 0 then begin
    //        ...
    RBResult := PackResDialog(CurrPath);
  end;
  If RBResult <> '' then begin
    //   .
    // .
    SetCurrPath(CurrPath);
    ShowFileList;
  end;
end;

procedure TTextEdF.WordWrapCBClick(Sender: TObject);
begin
  TxtEdDlgWordWrap := WordWrapCB.Checked;

  //    -   .
  EditBox.WordWrap := TxtEdDlgWordWrap;

  If Showing = false then begin
    //     
    //"" .
    AnIniFil.Load(IniFilPath);  // .
    AnIniFil.WriteBool('MainSettings','TxtEdDlgWordWrap',TxtEdDlgWordWrap);
    AnIniFil.Save(IniFilPath);  // .    :)
  end;
end;

Procedure TTextEdF.OpenSelectedRtf;
begin
  //   .
  OpenedTxt := CurrPath+FileListBox.Selected.Caption;
  If FSObjectExists(OpenedTxt) = true then begin
    // . .  -     .
    FileNameEd.Text := FileListBox.Selected.Caption;
    EditBox.PlainText := false;
    RtfModeRB.Checked := true;
    EditBox.Lines.LoadFromFile(OpenedTxt);
  end;
end;

procedure TTextEdF.TxtModeRBClick(Sender: TObject);
begin
  If TxtModeRB.Checked then begin
    EditBox.PlainText := true;
  end;
end;

procedure TTextEdF.RtfModeRBClick(Sender: TObject);
begin
  If RtfModeRB.Checked then begin
    EditBox.PlainText := false;
  end;
end;

procedure TTextEdF.FileListBoxEdited(Sender: TObject; Item: TListItem;
  var S: String);
begin
  S := Item.Caption;
  FLB_Editing := false;
end;

procedure TTextEdF.FileListBoxKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
var
  Done : boolean;
  
begin
  //   .

  If not FLB_Editing then begin
    //       .
    
    If Key = 46 then begin

      Done := false;

      //Del.   -   :).     .
      If FileListBox.ItemIndex <> -1 then begin
        //  .

        If (FileListBox.Selected.ImageIndex = 2) or (FileListBox.Selected.ImageIndex = 4) or (FileListBox.Selected.ImageIndex = 5) then begin
          // .
          If MessageBox(Application.Handle,PChar('    '+#13+CurrPath+FileListBox.Selected.Caption+'  ?'+#13+'  ,   !'),'!',MB_YESNO or MB_ICONWARNING) = ID_YES then begin
            If FileExists(CurrPath+FileListBox.Selected.Caption) = true then begin
              //  .
              SetFileAttributes(PChar(CurrPath+FileListBox.Selected.Caption),$00000020);
              DeleteFile(CurrPath+FileListBox.Selected.Caption);
              FileListBox.Selected.Delete;
              Done := true;
            end;
          end;
        end;

        If Done = false then begin
          If FileListBox.Selected.ImageIndex = 1 then begin
            // .
            If MessageBox(Application.Handle,PChar('    '+#13+CurrPath+FileListBox.Selected.Caption+'\   ?'+#13+'  ,   !'),'!',MB_YESNO or MB_ICONWARNING) = ID_YES then begin
              If DirectoryExists(CurrPath+FileListBox.Selected.Caption+'\') = true then begin
                // .
                DelForm.DelPath := CurrPath+FileListBox.Selected.Caption+'\';
                DelForm.ShowModal;
                FileListBox.Selected.Delete;
              end;
            end;
          end;
        end;  

      end;

    end;

  end;
end;

procedure TTextEdF.FileListBoxEditing(Sender: TObject; Item: TListItem;
  var AllowEdit: Boolean);
begin
  FLB_Editing := true;
end;

procedure TTextEdF.FileListBoxExit(Sender: TObject);
begin
  FLB_Editing := false;
end;

end.














